subroutine CalcEnergy(Pos, M, k, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: k, L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: r0 = 1.
    !======== YOUR CODE HERE ========
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12
    real(8) :: rc2, Shift, rij_s
    integer :: i, j
    !Compute the Lennard-Jones shift value before the pairwise loop
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
    PEnergy = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            if (j == i + 1 .and. mod(j, M) > 0) then
                !this is a bonded interaction;
        		!use the harmonic bond potential here
                rij = Pos(j,:) - Posi
                rij = rij - L * dnint(rij / L)
                rij_s = sqrt(sum(rij * rij))
        	PEnergy = PEnergy + (k * 0.5) * (rij_s-r0) ** 2.
            else
                !this is a nonbounded interaction;
            	!use the Lennard-Jones or electrostatic potential here
            	rij = Pos(j,:) - Posi
            	rij = rij - L * dnint(rij / L)
		rij_s = sqrt(sum(rij * rij))
            	!Compute only the squared distance and compare to the square cut                
            	d2 = sum(rij * rij)
            	if (d2 > rc2) then
            	    cycle
            	end if
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                !Calculate the PEnergy, U
		if (mod(j, M) == 0 .and. mod(i, M) == 0.) then 
		! this is an Lennard Jones and electrostatic interaction
                    PEnergy = PEnergy + 4. * (id12 - id6) + ((1000./rij_s) * exp(-0.5 * rij_s)) + Shift
		else
		    PEnergy = PEnergy + 4. * (id12 - id6)
		end if
            end if
    	enddo
    enddo   
end subroutine

subroutine CalcEnergyForces(Pos, M, k, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: k, L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: r0 = 1.
    !======== YOUR CODE HERE ========
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d2, id2, id6, id12
    real(8) :: rc2, Shift, rij_s
    integer :: i, j
    !Compute the Lennard-Jones shift value before the pairwise loop
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
    PEnergy = 0.
    Forces = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            if (j == i + 1 .and. mod(j, M) > 0) then
            	!this is a bonded interaction;
        		!use the harmonic bond potential here
        		rij = Pos(j,:) - Posi
        		rij = rij - L * dnint(rij / L)
        		rij_s = sqrt(sum(rij * rij))
        		PEnergy = PEnergy + (k * 0.5) * (rij_s-r0) ** 2.
        		Fij = k * (rij_s - r0) * rij / rij_s
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
    	    else
        		!this is a nonbounded interaction;
        		!use the Lennard-Jones potential here
    	        rij = Pos(j,:) - Posi
            	rij = rij - L * dnint(rij / L)
		rij_s = sqrt(sum(rij * rij))
                !Compute only the squared distance and compare to the square cut                
        		d2 = sum(rij * rij)
        		if (d2 > rc2) then
        		    cycle
        		end if
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                !Calculate the PEnergy, U, and Force from W,the virial
		if (mod(j, M) == 0 .and. mod(i, M) == 0.) then
		! this is an Lennard Jones and electrostatic interaction
            	    PEnergy = PEnergy + 4. * (id12 - id6) + ((1000. / rij_s) * exp(-0.5 * rij_s)) + Shift
		    Fij = rij * ((-48. * id12 + 24. * id6) * id2) + (-1000./(rij_s*rij_s*rij_s) -500./(rij_s*rij_s)) * rij * (exp(-0.5 * rij_s))
                    Forces(i,:) = Forces(i,:) + Fij
                    Forces(j,:) = Forces(j,:) - Fij    
		else
		    PEnergy = PEnergy + 4. * (id12 - id6)
    	            Fij = rij * ((-48. * id12 + 24. * id6) * id2) 
               	    Forces(i,:) = Forces(i,:) + Fij
                    Forces(j,:) = Forces(j,:) - Fij    
		end if
    	    end if
    	enddo
    enddo
end subroutine

subroutine VVIntegrate(Pos, Vel, Accel, M, k, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: k, L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, k, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine


subroutine Histogram(Pos, L, M, count, Bins, Dim, NAtom)
    implicit none
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L
    integer, intent(in) :: M, Bins, Dim, NAtom
    integer, intent(inout), dimension(0:Bins-1) :: count
!f2py intent(in,out,inplace) :: count
    real(8), dimension(Dim) :: Posi, rij
    integer :: i, j, k
    real(8) :: delta, d
    !Compute the histogram counts
    delta=real(L)/real(Bins)	
    do i = 0, NAtom - 1
        Posi=Pos(i,:)
	do j = i + 1, NAtom - 1	
	    rij=Pos(j,:) - Posi
	    rij=rij - L * dnint(rij/L)
	    d=sqrt(sum(rij * rij))
	    do k=0, Bins - 1
	        if (d < (delta*(k+1)) .and. delta*k <= d) then
		    count(k)=count(k)+1
		end if
	    enddo	
	enddo
    enddo
end subroutine